@@ -1,23 +1,6 @@
+// SPDX-License-Identifier: MIT
 /*
 Copyright 2011 by Matthieu Boutier and Juliusz Chroboczek
-
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-THE SOFTWARE.
 */
 
 /* FRR's includes */
@@ -139,7 +122,7 @@ DEFUN (debug_babel,
     for(i = 0; debug_type[i].str != NULL; i++) {
         if (strncmp (debug_type[i].str, argv[2]->arg,
                      debug_type[i].str_min_len) == 0) {
-            debug |= debug_type[i].type;
+            SET_FLAG(debug, debug_type[i].type);
             return CMD_SUCCESS;
         }
     }
@@ -169,7 +152,7 @@ DEFUN (no_debug_babel,
     for (i = 0; debug_type[i].str; i++) {
         if (strncmp(debug_type[i].str, argv[3]->arg,
                     debug_type[i].str_min_len) == 0) {
-            debug &= ~debug_type[i].type;
+            UNSET_FLAG(debug, debug_type[i].type);
             return CMD_SUCCESS;
         }
     }
@@ -195,16 +178,18 @@ debug_babel_config_write (struct vty * vty)
         lines++;
     }
     else
+    {
         for (i = 0; debug_type[i].str != NULL; i++)
-            if
-            (
-                debug_type[i].type != BABEL_DEBUG_ALL
-                && CHECK_FLAG (debug, debug_type[i].type)
-            )
+        {
+            if (debug_type[i].type != BABEL_DEBUG_ALL
+                && CHECK_FLAG (debug, debug_type[i].type))
             {
                 vty_out (vty, "debug babel %s\n", debug_type[i].str);
                 lines++;
             }
+        }
+    }
+
     if (lines)
     {
         vty_out (vty, "!\n");
