@@ -22,13 +22,19 @@
 #include "northbound_db.h"
 #include "lib/northbound_cli_clippy.c"
 
-struct debug nb_dbg_cbs_config = {0, "Northbound callbacks: configuration"};
-struct debug nb_dbg_cbs_state = {0, "Northbound callbacks: state"};
-struct debug nb_dbg_cbs_rpc = {0, "Northbound callbacks: RPCs"};
-struct debug nb_dbg_cbs_notify = {0, "Northbound callbacks: notifications"};
-struct debug nb_dbg_notif = {0, "Northbound notifications"};
-struct debug nb_dbg_events = {0, "Northbound events"};
-struct debug nb_dbg_libyang = {0, "libyang debugging"};
+struct debug nb_dbg_cbs_config = { 0, "debug northbound callbacks configuration",
+				   "Northbound callbacks: configuration" };
+struct debug nb_dbg_cbs_state = { 0, "debug northbound callbacks state",
+				  "Northbound callbacks: state" };
+struct debug nb_dbg_cbs_rpc = { 0, "debug northbound callbacks rpc",
+				"Northbound callbacks: RPCs" };
+struct debug nb_dbg_cbs_notify = { 0, "debug northbound callbacks notify",
+				   "Northbound callbacks: notifications" };
+struct debug nb_dbg_notif = { 0, "debug northbound notifications",
+			      "Northbound notifications" };
+struct debug nb_dbg_events = { 0, "debug northbound events",
+			       "Northbound events" };
+struct debug nb_dbg_libyang = { 0, "debug northbound libyang", "libyang" };
 
 struct nb_config *vty_shared_candidate_config;
 static struct event_loop *master;
@@ -1380,7 +1386,7 @@ static int nb_cli_show_transactions(struct vty *vty)
 
 		table = ttable_dump(tt, "\n");
 		vty_out(vty, "%s\n", table);
-		XFREE(MTYPE_TMP, table);
+		XFREE(MTYPE_TMP_TTABLE, table);
 	} else
 		vty_out(vty, "No configuration transactions to display.\n\n");
 
@@ -1661,7 +1667,7 @@ DEFPY (show_yang_module,
 
 		table = ttable_dump(tt, "\n");
 		vty_out(vty, "%s\n", table);
-		XFREE(MTYPE_TMP, table);
+		XFREE(MTYPE_TMP_TTABLE, table);
 	} else
 		vty_out(vty, "No YANG modules to display.\n\n");
 
@@ -1771,7 +1777,7 @@ DEFPY (show_yang_module_translator,
 
 		table = ttable_dump(tt, "\n");
 		vty_out(vty, "%s\n", table);
-		XFREE(MTYPE_TMP, table);
+		XFREE(MTYPE_TMP_TTABLE, table);
 	} else
 		vty_out(vty, "No YANG module translators to display.\n\n");
 
@@ -1842,37 +1848,6 @@ DEFPY (rollback_config,
 }
 
 /* Debug CLI commands. */
-static struct debug *nb_debugs[] = {
-	&nb_dbg_cbs_config, &nb_dbg_cbs_state, &nb_dbg_cbs_rpc,
-	&nb_dbg_cbs_notify, &nb_dbg_notif,     &nb_dbg_events,
-	&nb_dbg_libyang,
-};
-
-static const char *const nb_debugs_conflines[] = {
-	"debug northbound callbacks configuration",
-	"debug northbound callbacks state",
-	"debug northbound callbacks rpc",
-	"debug northbound callbacks notify",
-	"debug northbound notifications",
-	"debug northbound events",
-	"debug northbound libyang",
-};
-
-DEFINE_HOOK(nb_client_debug_set_all, (uint32_t flags, bool set), (flags, set));
-
-static void nb_debug_set_all(uint32_t flags, bool set)
-{
-	for (unsigned int i = 0; i < array_size(nb_debugs); i++) {
-		DEBUG_FLAGS_SET(nb_debugs[i], flags, set);
-
-		/* If all modes have been turned off, don't preserve options. */
-		if (!DEBUG_MODE_CHECK(nb_debugs[i], DEBUG_MODE_ALL))
-			DEBUG_CLEAR(nb_debugs[i]);
-	}
-
-	hook_call(nb_client_debug_set_all, flags, set);
-}
-
 DEFPY (debug_nb,
        debug_nb_cmd,
        "[no] debug northbound\
@@ -1895,8 +1870,13 @@ DEFPY (debug_nb,
        "libyang debugging\n")
 {
 	uint32_t mode = DEBUG_NODE2MODE(vty->node);
+	bool all = false;
 
-	if (cbs) {
+	/* no specific debug --> act on all of them */
+	if (strmatch(argv[argc - 1]->text, "northbound"))
+		all = true;
+
+	if (cbs || all) {
 		bool none = (!cbs_cfg && !cbs_state && !cbs_rpc && !cbs_notify);
 
 		if (none || cbs_cfg)
@@ -1908,45 +1888,18 @@ DEFPY (debug_nb,
 		if (none || cbs_notify)
 			DEBUG_MODE_SET(&nb_dbg_cbs_notify, mode, !no);
 	}
-	if (notifications)
+	if (notifications || all)
 		DEBUG_MODE_SET(&nb_dbg_notif, mode, !no);
-	if (events)
+	if (events || all)
 		DEBUG_MODE_SET(&nb_dbg_events, mode, !no);
-	if (libyang) {
+	if (libyang || all) {
 		DEBUG_MODE_SET(&nb_dbg_libyang, mode, !no);
 		yang_debugging_set(!no);
 	}
 
-	/* no specific debug --> act on all of them */
-	if (strmatch(argv[argc - 1]->text, "northbound")) {
-		nb_debug_set_all(mode, !no);
-		yang_debugging_set(!no);
-	}
-
 	return CMD_SUCCESS;
 }
 
-DEFINE_HOOK(nb_client_debug_config_write, (struct vty *vty), (vty));
-
-static int nb_debug_config_write(struct vty *vty)
-{
-	for (unsigned int i = 0; i < array_size(nb_debugs); i++)
-		if (DEBUG_MODE_CHECK(nb_debugs[i], DEBUG_MODE_CONF))
-			vty_out(vty, "%s\n", nb_debugs_conflines[i]);
-
-	hook_call(nb_client_debug_config_write, vty);
-
-	return 1;
-}
-
-static struct debug_callbacks nb_dbg_cbs = {.debug_set_all = nb_debug_set_all};
-static struct cmd_node nb_debug_node = {
-	.name = "northbound debug",
-	.node = NORTHBOUND_DEBUG_NODE,
-	.prompt = "",
-	.config_write = nb_debug_config_write,
-};
-
 void nb_cli_install_default(int node)
 {
 	_install_element(node, &show_config_candidate_section_cmd);
@@ -2007,9 +1960,14 @@ void nb_cli_init(struct event_loop *tm)
 	/* Initialize the shared candidate configuration. */
 	vty_shared_candidate_config = nb_config_new(NULL);
 
-	debug_init(&nb_dbg_cbs);
+	debug_install(&nb_dbg_cbs_config);
+	debug_install(&nb_dbg_cbs_state);
+	debug_install(&nb_dbg_cbs_rpc);
+	debug_install(&nb_dbg_cbs_notify);
+	debug_install(&nb_dbg_notif);
+	debug_install(&nb_dbg_events);
+	debug_install(&nb_dbg_libyang);
 
-	install_node(&nb_debug_node);
 	install_element(ENABLE_NODE, &debug_nb_cmd);
 	install_element(CONFIG_NODE, &debug_nb_cmd);
 
