@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * SRC-DEST Routing Table
  *
@@ -5,20 +6,6 @@
  *                       Open Source Routing / NetDEF Inc.
  *
  * This file is part of FRRouting (FRR)
- *
- * FRR is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * FRR is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -30,7 +17,7 @@
 #include "table.h"
 #include "printfrr.h"
 
-DEFINE_MTYPE_STATIC(LIB, ROUTE_SRC_NODE, "Route source node")
+DEFINE_MTYPE_STATIC(LIB, ROUTE_SRC_NODE, "Route source node");
 
 /* ----- functions to manage rnodes _with_ srcdest table ----- */
 struct srcdest_rnode {
@@ -306,14 +293,29 @@ const char *srcdest_rnode2str(const struct route_node *rn, char *str, int size)
 	return srcdest2str(dst_p, (const struct prefix_ipv6 *)src_p, str, size);
 }
 
-printfrr_ext_autoreg_p("RN", printfrr_rn)
-static ssize_t printfrr_rn(char *buf, size_t bsz, const char *fmt,
-			   int prec, const void *ptr)
+printfrr_ext_autoreg_p("RN", printfrr_rn);
+static ssize_t printfrr_rn(struct fbuf *buf, struct printfrr_eargs *ea,
+			   const void *ptr)
 {
 	const struct route_node *rn = ptr;
 	const struct prefix *dst_p, *src_p;
+	char cbuf[PREFIX_STRLEN * 2 + 6];
+
+	if (!rn)
+		return bputs(buf, "(null)");
 
 	srcdest_rnode_prefixes(rn, &dst_p, &src_p);
-	srcdest2str(dst_p, (const struct prefix_ipv6 *)src_p, buf, bsz);
-	return 2;
+	srcdest2str(dst_p, (const struct prefix_ipv6 *)src_p,
+		    cbuf, sizeof(cbuf));
+	return bputs(buf, cbuf);
+}
+
+struct route_table *srcdest_srcnode_table(struct route_node *rn)
+{
+	if (rnode_is_dstnode(rn)) {
+		struct srcdest_rnode *srn = srcdest_rnode_from_rnode(rn);
+
+		return srn->src_table;
+	}
+	return NULL;
 }
